"""visaweb URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.10/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""
from django.conf.urls import url, include
from rest_framework import routers

from . import views

router = routers.DefaultRouter()
# router.register(r'visa', views.UserViewSet)
router.register(r'users', views.UserViewSet)
router.register(r'groups', views.GroupViewSet)
router.register(r'assets', views.AssetViewSet)



urlpatterns = [
    # url(r'^$', views.health, name='health'),

    # ex: /polls/
    url(r'^$', views.index, name='index'),
    # url(r'^$', TemplateView.as_view(template_name= 'index.html'), name='home'),
    # url(r'^$', 'django.contrib.staticfiles.views.serve', kwargs={'path': 'index.html', 'document_root': '/static/'}),

    url(r'^', include(router.urls)),
    url(r'^api-auth/', include('rest_framework.urls', namespace='rest_framework')),

    url(r'^devicemanager/attach$', views.devicemanager_attach, name='devicemanager_attach'),
    url(r'^devicemanager/active', views.devicemanager_active, name='devicemanager_active'),
    url(r'^devicemanager/inactive$', views.devicemanager_inactive, name='devicemanager_inactive'),
    url(r'^devicemanager/unknown$', views.devicemanager_unknown, name='devicemanager_unknown'),
    url(r'^visa/list$', views.visa_list, name='visa_list'),
    url(r'^visa/alias', views.visa_alias, name='visa_alias'),
    url(r'^visa/read$', views.visa_read, name='visa_read'),
    url(r'^visa/write', views.visa_write, name='visa_write'),
    url(r'^visa/query', views.visa_query, name='visa_query'),

    # ex: /polls/5/
    url(r'^(?P<asset_id>[0-9]+)/$', views.detail, name='detail'),
    # ex: /polls/5/results/
    url(r'^(?P<asset_id>[0-9]+)/results/$', views.results, name='results'),
    # ex: /polls/5/vote/
    url(r'^(?P<question_id>[0-9]+)/vote/$', views.vote, name='vote'),
]


# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-17 18:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('visaweb', '0003_auto_20170417_1832'),
    ]

    operations = [
        migrations.AddField(
            model_name='command',
            name='name',
            field=models.CharField(default='trace', max_length=200),
            preserve_default=False,
        ),
    ]



from _ast import alias

import numpy
import pyvisa as visa
import logging
import datetime
import json
import threading

class ScVisaError(Exception):
    def __init__(self, message):
        self.ts = str(datetime.datetime.now())
        self.message = message
    def __str__(self):
        return '%s: %s' % (self.ts, self.message)

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)

class DeviceManager():
    def __init__(self, visa_library):
        """
        Constructor.
        :param visa_library: PyVISA backend library. 
        """
        self.lock = threading.Lock()
        self.rm = visa.ResourceManager(visa_library)

        self.lock.acquire()
        try:
            logging.info(self.rm.list_resources())
            self.devices = {}
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))
        finally:
            self.lock.release()

    def alias(self, resourceMap):
        """
        VISA alias command to make a friendly name to a resource.
        :param resourceMap: Dictionary, keys are the friendly name, values are the VISA connection strings.
        :return: Count of connected elements.
        """
        index = 0
        self.lock.acquire()
        try:
            logging.info('Connectiong devices ...')
            for key, value in resourceMap.items():
                try:
                    if any(value in s for s in self.rm.list_resources()):
                        res = self.rm.open_resource(value)
                        logging.info('<' + key + '> : ' + value)
                        self.devices[key] = res
                        index += 1
                except Exception as ex:
                    logging.exception("message")
            return index
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))
        finally:
            self.lock.release()

    def query(self, alias, command):
        """
        VISA query command.
        :param alias: Asset friendly name.
        :param command: VISA query command.
        :return: Parsed result of the VISA query.
        """
        self.lock.acquire()
        try:
            logging.info("dm.query" + alias + ":" + command)
            resource = self.devices[alias]
            # return resource.query_ascii_values(command, container=numpy.array, separator=',')
            return resource.query(command)
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))
        finally:
            self.lock.release()

    def query_ascii(self, alias, command):
        """
        VISA query command to ASCII nd-array.
        :param alias: Asset friendly name.
        :param command: VISA query command.
        :return: Parsed result of the VISA query.
        """
        self.lock.acquire()
        try:
            logging.info("dm.query_ascii" + alias + ":" + command)
            resource = self.devices[alias]
            return resource.query_ascii_values(command, container=numpy.array, separator=',')
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))
        finally:
            self.lock.release()

    def read(self, alias, command):
        """
        VISA read command
        :param alias: Asset friendly name.
        :param command: VISA read command.
        :return: Parsed result of VISA command.
        """
        self.lock.acquire()
        try:
            logging.info("dm.read" + alias + ":" + command)
            resource = self.devices[alias]
            return resource.read(command)
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))
        finally:
            self.lock.release()

    def write(self, alias, command):
        """
        VISA write command.
        :param alias: Asset friendly name.
        :param command: VISA write command.
        """
        self.lock.acquire()
        try:
            logging.info("dm.write" + alias + ":" + command)
            resource = self.devices[alias]
            resource.write(command)
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))
        finally:
            self.lock.release()

    def list(self):
        """
        List discoverable VISA resources.
        :return: Array of the connection strings.
        """
        self.lock.acquire()
        try:
            logging.info("dm.list")
            return self.rm.list_resources()
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))
        finally:
            self.lock.release()


from __future__ import unicode_literals

from django.db import models

# Create your models here.

class Entity(models.Model):
    display_name = models.CharField(max_length=200)
    description = models.CharField(max_length=200, null=True, blank=True)

    def __str__(self):
        return self.display_name.encode('utf8')

class EnumerationType(Entity):
    def choices(self):
        return self.choice_set.all()

class EnumerationChoice(models.Model):
    value = models.CharField(max_length=200)
    owner = models.ForeignKey(EnumerationType, on_delete=models.CASCADE)

    def __str__(self):
        return self.value.encode('utf8')




class Asset(Entity):
    alias = models.CharField(max_length=200, null=True, blank=True, default=None)
    address = models.CharField(max_length=200)

    # def __str__(self):
    #     return self.alias

class Command(models.Model):
    code = models.CharField(max_length=200)
    subject = models.ForeignKey(Asset, related_name='actions', on_delete=models.CASCADE)
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.code.encode('utf8')

class Property(Entity):
    name = models.CharField(max_length=200)
    owner = models.ForeignKey(Asset, related_name='properties', on_delete=models.CASCADE)
    unit_choices = models.ForeignKey(EnumerationType, on_delete=models.CASCADE, null=True, blank=True, default = None)
    default_unit = models.ForeignKey(EnumerationChoice, on_delete=models.CASCADE, null=True, blank=True, default = None)
    type = models.CharField(max_length=200, default='string', choices=((u'string', u'string'), (u'interval', u'interval'), (u'choice', u'choice')))
    getter_command = models.OneToOneField(Command, related_name='getter_command', on_delete=models.CASCADE, null=True, blank=True)
    setter_command = models.OneToOneField(Command, related_name='setter_command', on_delete=models.CASCADE, null=True, blank=True)

    # def __str__(self):
    #     return self.name

class StringProperty(Property):
    default_value = models.CharField(max_length=200, null=True, blank=True)

    def __init__(self, *args, **kwargs):
        super(StringProperty, self).__init__(*args, **kwargs)
        StringProperty._meta.get_field('type').default = 'string'

class ChoiceProperty(Property):
    value_choices = models.ForeignKey(EnumerationType, on_delete=models.CASCADE)
    default_value = models.ForeignKey(EnumerationChoice, on_delete=models.CASCADE, null=True, blank=True, default = None)


    def __init__(self, *args, **kwargs):
        super(ChoiceProperty, self).__init__(*args, **kwargs)
        ChoiceProperty._meta.get_field('type').default = 'choice'

    def get_value(self):
        return EnumerationChoice.get(id=self.choice)

class IntervalProperty(Property):
    minimum = models.FloatField()
    maximum = models.FloatField()
    default_value = models.FloatField()

    def __init__(self, *args, **kwargs):
        super(IntervalProperty, self).__init__(*args, **kwargs)
        IntervalProperty._meta.get_field('type').default = 'interval'

    def get_value(self):
        return self.value

class RepositoryService:

    def assets(self):
        return Asset.objects.all()


"""
Django settings for rmd project.

Generated by 'django-admin startproject' using Django 1.10.6.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'a_48z)u+g@p91aem8g8zs^6qf+b5=8e9o(oq)y=z33^m8=@@0l'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'rest_framework',
    # 'channels',
    'visaweb.apps.VisawebConfig',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'rmd.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            'visaweb/static/visaweb',
            'visaweb/static/visaweb/bower_components',
            'visaweb/static/visaweb/images'
            'visaweb/static/visaweb/src'
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'rmd.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly',
        'rest_framework.permissions.DjangoModelPermissions',
    ]
}

# # In settings.py
# CHANNEL_LAYERS = {
#     "default": {
#         "BACKEND": "asgiref.inmemory.ChannelLayer",
#         "ROUTING": "myproject.routing.channel_routing",
#     },
# }

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = '/static/'



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-17 15:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Entity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('display_name', models.CharField(max_length=200)),
                ('description', models.CharField(blank=True, max_length=200, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='EnumerationChoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=200)),
            ],
        ),
        migrations.CreateModel(
            name='Asset',
            fields=[
                ('entity_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='visaweb.Entity')),
                ('alias', models.CharField(blank=True, default=None, max_length=200, null=True)),
                ('address', models.CharField(max_length=200)),
            ],
            bases=('visaweb.entity',),
        ),
        migrations.CreateModel(
            name='EnumerationType',
            fields=[
                ('entity_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='visaweb.Entity')),
            ],
            bases=('visaweb.entity',),
        ),
        migrations.CreateModel(
            name='Property',
            fields=[
                ('entity_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='visaweb.Entity')),
                ('name', models.CharField(max_length=200)),
                ('type', models.CharField(choices=[('string', 'string'), ('interval', 'interval'), ('choice', 'choice')], default='string', max_length=200)),
            ],
            bases=('visaweb.entity',),
        ),
        migrations.CreateModel(
            name='ChoiceProperty',
            fields=[
                ('property_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='visaweb.Property')),
            ],
            bases=('visaweb.property',),
        ),
        migrations.CreateModel(
            name='IntervalProperty',
            fields=[
                ('property_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='visaweb.Property')),
                ('minimum', models.FloatField()),
                ('maximum', models.FloatField()),
                ('default_value', models.FloatField()),
            ],
            bases=('visaweb.property',),
        ),
        migrations.CreateModel(
            name='StringProperty',
            fields=[
                ('property_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='visaweb.Property')),
                ('default_value', models.CharField(blank=True, max_length=200, null=True)),
            ],
            bases=('visaweb.property',),
        ),
        migrations.AddField(
            model_name='property',
            name='default_unit',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='visaweb.EnumerationChoice'),
        ),
        migrations.AddField(
            model_name='property',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='properties', to='visaweb.Asset'),
        ),
        migrations.AddField(
            model_name='property',
            name='unit_choices',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='visaweb.EnumerationType'),
        ),
        migrations.AddField(
            model_name='enumerationchoice',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='visaweb.EnumerationType'),
        ),
        migrations.AddField(
            model_name='choiceproperty',
            name='default_value',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='visaweb.EnumerationChoice'),
        ),
        migrations.AddField(
            model_name='choiceproperty',
            name='value_choices',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='visaweb.EnumerationType'),
        ),
    ]



from rest_framework import serializers

from django.contrib.auth.models import User, Group
from .models import Asset, Property

import assetadapter
class UserSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = User
        fields = ('url', 'username', 'email', 'groups')


class GroupSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Group
        fields = ('url', 'name')

class PropertySerializer(serializers.ModelSerializer):
    class Meta:
        model = Property
        fields = ('name', 'display_name', 'description', 'type')


class AssetListSerializer(serializers.ListSerializer):
    properties = PropertySerializer(many=True, read_only=True)

    class Meta:
        model = Asset
        fields = ('alias', 'address', 'id', 'properties', 'display_name', 'description')

class AssetSerializer(serializers.ModelSerializer):
    properties = PropertySerializer(many=True, read_only=True)

    class Meta:
        model = Asset
        fields = ('alias', 'address', 'id', 'properties', 'display_name', 'description')